#-*- coding: UTF-8 -*-

#***********************************************
# R2rpm
#
# Class file for the class Build
#
#
# Made the 07th March 2010
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#***********************************************

import subprocess, os
from R2rpmError import *

class Build:
    """ Build class
            Builds the specfile and handles the errors
    """
    
    def __init__(self):
        self.specname = ''
        self.outcode = None

    def build(self, specname):
        ''' Builds the given specfile and returns the errors if any '''
        print "* Building the rpm (this may take some time)"
        cmd = "   rpmbuild -ba %s > build-%s.log 2>&1" %(specname, specname)
        print cmd
        self.outcode = subprocess.call(cmd , shell=True)
        #print self.outcode
    
    def parseError(self, specname):
        ''' Parses the build.log to extract the error 
                Generates a file list (file installed but not found)
                Throw an error Package not found
                Throw an error Build Time error
        '''
        
        f = open("build-%s.log" %specname, 'r')
        log = f.read()
        f.close
        logs = log.split("\n")
        
        flagfile = False
        flagdep = False
        self.files = [] # stores the files for the %file section
        self.deps = [] # stores missing dependencies
        for row in logs:
            if row == "":
                flagfile = False
            if flagfile:
                self.files.append(row)
            if flagdep:
                self.deps.append( row.split('is needed')[0].replace('\t','').strip() )
            if row == 'error: Installed (but unpackaged) file(s) found:':
                flagfile = True
            if row == 'error: Failed build dependencies:':
                flagdep = True
        
        if len(self.deps) != 0:
            dep = ", ".join(self.deps)
            raise BuildDepencenciesError("Dependencies missing: %s"%dep)
        
        if len(self.deps) == 0 and len(self.files) == 0 and self.outcode == 1:
            log = os.getcwd() + "/build-%s.log" %specname
            raise BuildError("An error occured at build time, see the log in %s"%log)
        
        return self.files
        
