#!/usr/bin/python
#-*- coding: UTF-8 -*-

#***********************************************
# R2rpm
#
# R2rpm generates RPM using the R2spec library
# building the RPM and adjusting the spec accordingly
#
#
# Made the 07th March 2010
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#***********************************************

from Package import *
from optparse import OptionParser
import getopt, sys, re
from R2spec import Spec


USAGE ="""
This program is designed to create RPM from a source file or a url using R2spec.
Usage:  R2spec -u <url> 
        R2spec -s <source>
        R2spec -s folder/folder/<source>

You can also use the following options to add the correct Source0 in the spec file:
  --bioc                For Bioconductor libraries
  --cran                For libraries coming from the CRAN repository
  --rforge              For libraries coming from the r-forge.net
  --rproject            For libraries coming from the r-forge.r-project.org

The source can be in the same directory or in a subdirectory.

The information about the packager:
- Fill /etc/R2spec.conf 
- Copy and fill /etc/R2spec to ~/.R2spec.conf
- Use the option -n/--name and -e/--email to set the corresponding values
"""

 
#################################################
### argument
# retrieve the arguments given and show the help when needed
def getArgument():
    ''' Handle the parameters'''
    parser = OptionParser(version="%prog 0.1.0")
    parser.add_option("-u", "--url", dest="url",
                  help="Url of the R library")
    parser.add_option("-s", "--source", dest="source",
                  help="The source file (tarball) of the R library")
                  
    parser.add_option("--bioc", dest="bioc", default=False, action="store_true",
                  help="For Bioconductor libraries, will add the correct Source0 in the spec file")
    parser.add_option("--cran", dest="cran", default=False, action="store_true",
                  help="For libraries coming from the CRAN repository, will add the correct Source0 in the spec file")
    parser.add_option("--rforge", dest="rforge", default=False, action="store_true",
                  help="For libraries coming from the r-forge.net repository, will add the correct Source0 in the spec file")
    parser.add_option("--rproject", dest="rproject", default=False, action="store_true",
                  help="For libraries coming from the r-forge.r-project.org repository, will add the correct Source0 in the spec file")
                  
    parser.add_option("-c", "--copyFile", dest="copyFile", default=False, action="store_true",
                  help="Copy directly the file to the SOURCES folder without asking")
    parser.add_option("-d", "--dependencies", dest="dependencies", default=False, action="store_true",
                  help="Build the dependencies is any missing (default is False)")
                  
    parser.add_option("-n", "--name", dest="name",
                  help="Name of the packager that will be used in the spec file")
    parser.add_option("-e", "--email", dest="email",
                  help="Email of the packager that will be used in the spec file")
    parser.add_option("-f", "--force", dest="force", default=False, action="store_true",
                  help="Create the spec file even if a file of the same name exists allready in the working directory")
    (options, args) = parser.parse_args()

    return (options.source, options.url, \
        options.bioc, options.cran, options.rforge, options.rproject, \
        options.copyFile, options.dependencies, options.name, \
        options.email, options.force)
################################################


#################################################
### main
# Do the job :)
class R2rpm:
    """ R2rpm class
            Enables to use this code from another script
    """
    
    def __init__(self):
        ''' Instanciate the default values for the attributes '''
        self.build = []
    
    def main(self, source, url, bioc, cran, rforge, rproject, copyFile, dependencies, name, email, force):
        ''' Performs the work'''
        
        # Generate the spec file
        from R2spec import R2spec
        from R2spec import Package
        
        specdir = Package().getRPMTopDirectory()
        if specdir != None:
            os.chdir(specdir + "/SPECS/")
        specname = R2spec().main(source, url, bioc, cran, rforge, rproject, True, name, email, True)
        specname = specname + ".spec"
        self.build.append(specname)
        
        print "*"*50
        print "** Spec generated, starting RPM construction"
        # Remove the content of the %file section
        spec = Spec()
        spec.readSpec(specname)
        spec.commentFiles()
        spec.writeSpecTosystem(specname)
        self.generateRpm(spec, specname)
        
        
    def generateRpm(self, spec, specname):
        ''' Generate the rpm '''
        from Build import Build
        from R2rpmError import *
        
        # Build
        b = Build()
        try:
            b.build(specname)
            files = b.parseError(specname)
            spec.generateFile(files)
            spec.writeSpecTosystem(specname)
            if not b.outcode:
                self.generateRpm(spec, specname)
        except BuildDepencenciesError, err:
            print "An error occured during the build"
            print err
            if dependencies:
                print "\nBuilding dependencies"
                for package in b.deps:
                    print package
            sys.exit(1)
        except BuildError, err:
            print "An error occured during the build"
            print err
            sys.exit(1)
        
        print "** RPM %s done **" %specname.split('.spec')[0]
            

# Start        
# Retrieve the given arguments
(source, url, bioc, cran, rforge, rproject, copyFile, dependencies, name, email, force) = getArgument()
R2rpm().main(source, url, bioc, cran, rforge, rproject, copyFile, dependencies, name, email, force)
